@@ -1,6 +1,8 @@
 from __future__ import annotations
+
 import dir_ops as do
 from parent_class import ParentClass
+from parent_class import ParentPluralList
 
 import datetime
 from typing import List, Any, Tuple
@@ -112,6 +114,7 @@ class BaseDir( ParentClass ):
 
     ##################
 
+    @ps.try_operation_wrap( debug = do.DEBUG )
     @do.inherited_instance_method
     def exists( self, *args, **kwargs ) -> bool:
         pass
@@ -173,7 +176,7 @@ class BaseDir( ParentClass ):
 
     ##################
 
-    def list_contents( self, *args, print_off: bool = True, **kwargs ):
+    def list_contents( self, *args, print_off: bool = False, **kwargs ):
 
         filenames = []
         filenames.extend( self.list_subfolders() )
@@ -233,20 +236,20 @@ class BaseDir( ParentClass ):
     
     ##################
 
-    def list_contents_Paths( self, block_dirs: bool = True, block_paths: bool = False ) -> BasePaths:
+    def list_contents_Paths( self, block_dirs: bool = True, block_paths: bool = False, **kwargs ) -> BasePaths:
 
         Paths_inst = self.PATHS_CLASS( **self.inherited_kwargs )
 
         # 1. Add all files
         if not block_paths:
-            paths = self.list_files()
+            paths = self.list_files( **kwargs )
 
             for path in paths:
                 Paths_inst._add( self.join_Path( path = path ) )
 
         # 2. Add all dirs
         if not block_dirs:
-            dirs = self.list_subfolders()
+            dirs = self.list_subfolders( **kwargs )
 
             for dir in dirs:
                 Paths_inst._add( self.join_Dir( path = dir ) )
@@ -254,14 +257,14 @@ class BaseDir( ParentClass ):
         return Paths_inst
 
 
-    def walk( self, folders_to_skip: List[str] = ['.git'] ) -> BasePaths:
+    def walk( self, folders_to_skip: List[str] = ['.git'], **kwargs ) -> BasePaths:
 
         """Walk through all the contents of the directory"""
 
         Paths_inst = self.PATHS_CLASS( **self.inherited_kwargs )
         Paths_inst._add( self )
 
-        Paths_under = self.list_contents_Paths( block_dirs = False, block_paths = False )
+        Paths_under = self.list_contents_Paths( block_dirs = False, block_paths = False, **kwargs )
 
         for Path_inst in Paths_under:
 
@@ -272,15 +275,15 @@ class BaseDir( ParentClass ):
             elif Path_inst.type_dir:
                 
                 if Path_inst.dirs[-1] not in folders_to_skip:
-                    Paths_inst.merge( Path_inst.walk( folders_to_skip = folders_to_skip ) )
+                    Paths_inst.merge( Path_inst.walk( folders_to_skip = folders_to_skip, **kwargs ) )
 
         return Paths_inst
 
-    def walk_contents_Paths( self, block_dirs: bool = True, block_paths: bool = False, folders_to_skip: List[str] = ['.git'] ) -> BasePaths:
+    def walk_contents_Paths( self, block_dirs: bool = True, block_paths: bool = False, folders_to_skip: List[str] = ['.git'], **kwargs ) -> BasePaths:
 
         """get all Paths and/or Dirs underneath the entire directory, optional params for returning paths and/or dirs"""
 
-        Paths_inst = self.walk( folders_to_skip=folders_to_skip )
+        Paths_inst = self.walk( folders_to_skip=folders_to_skip, **kwargs )
         keep_Paths = self.PATHS_CLASS( **self.inherited_kwargs )
 
         for Path_inst in Paths_inst:
@@ -292,13 +295,13 @@ class BaseDir( ParentClass ):
 
         return keep_Paths
 
-    def get_unique_Path( self, filename: str ) -> str:
+    def get_unique_Path( self, filename: str, **list_contents_Paths_kwargs ) -> str:
 
         """finds a unique Path for the proposed filename based on the contents of the Directory
         if file.txt already exists in the dir, return file1.txt or file2.txt, etc  """
 
         # 
-        Paths_in_Dir = self.list_contents_Paths( block_dirs=True, block_paths=False )
+        Paths_in_Dir = self.list_contents_Paths( block_dirs=True, block_paths=False, **list_contents_Paths_kwargs )
         filenames = [ P.filename for P in Paths_in_Dir ]
 
         filename_Path = self.join_Path( path = filename )
@@ -475,20 +478,19 @@ class BasePath( BaseDir ):
 
 
 
-class BaseDirs( ParentClass ):
+class BaseDirs( ParentPluralList ):
 
     inherited_kwargs = {}
 
     def __init__( self, *args, given_Dirs = [], given_Paths = [], **kwargs ):
 
-        ParentClass.__init__( self )
+        ParentPluralList.__init__( self, att = 'Objs' )
         self.DIR_CLASS = BaseDir
         self.PATH_CLASS = BasePath
         self.DIRS_CLASS = BaseDirs
         self.PATHS_CLASS = BasePaths
 
-        self.Dirs = []
-        self.Objs = self.Dirs # Make an Alias
+        self.Dirs = self.Objs # Make an Alias
 
         for D in given_Dirs:
             self._add( D )
@@ -506,55 +508,6 @@ class BaseDirs( ParentClass ):
         """returns boolean if Object is a Dir"""
         return isinstance( Object, BaseDirs )
 
-    def __len__( self ):
-
-        return len( self.Objs )
-
-    def __iter__( self ):
-
-        self.i = -1
-        return self
-
-    def __next__( self ):
-
-        self.i += 1
-
-        if self.i < len(self):
-            return self.Objs[self.i]
-        else:
-            raise StopIteration
-
-    def __contains__( self, Obj_to_check: Any ) -> bool:
-
-        """returns the boolean value for Dir/Path Obj being contained in the list of Objects"""
-
-        for Obj in self:
-            if Obj == Obj_to_check:
-                return True
-        return False
-
-    def _add( self, new_Obj: Any ) -> None:
-
-        """add a new Object to the list of Objects"""
-
-        self.Objs.append( new_Obj )
-
-    def print_imp_atts( self, print_off = True ):
-
-        string = self._print_imp_atts_helper( print_off = False ) + '\n'
-        string += 'Dirs:\n'
-
-        for D in self:
-            string += D.print_one_line_atts( print_off = False ) + '\n'
-
-        string = string [:-1]
-        return self.print_string( string, print_off = print_off )
-
-    def print_one_line_atts(self, print_off = True, leading_string = '\t' ):
-
-        self.len_Dirs = len(self)
-        return self._print_one_line_atts_helper( atts = ['type','len_Dirs'], print_off = print_off, leading_string = leading_string )
-
     def join_Dir( self, Dir_inst: BaseDir ) -> BasePaths:
 
         """Joins Dir_inst to each Dir/Path contained in the Object"""
@@ -597,10 +550,9 @@ class BasePaths( BaseDirs ):
 
     def __init__ ( self, *args, **kwargs ):
 
-        self.Paths = []
-        self.Objs = self.Paths # Make an Alias
-
         BaseDirs.__init__( self, **kwargs )
+        self.Paths = self.Objs # Make an Alias
+
         self.DIR_CLASS = BaseDir
         self.PATH_CLASS = BasePath
         self.DIRS_CLASS = BaseDirs
@@ -617,23 +569,6 @@ class BasePaths( BaseDirs ):
         """returns boolean if Object is a Dir"""
         return isinstance( Object, BasePaths )
 
-    def print_imp_atts( self, print_off = True ):
-
-        string = self._print_imp_atts_helper( print_off = False ) + '\n'
-        string += 'Paths:\n'
-
-        for P in self:
-            string += P.print_one_line_atts( print_off = False ) + '\n'
-
-        string = string [:-1]
-        return self.print_string( string, print_off = print_off )
-
-    def print_one_line_atts(self, print_off = True, leading_string = '\t' ):
-
-        self.len_Paths = len(self)
-        return self._print_one_line_atts_helper( atts = ['type','len_Paths'], print_off = print_off, leading_string = leading_string )
-
-
     def get_rels( self, Dir_inst ) -> BasePaths:
 
         """Given a Dir object, find the relative Paths from Dir to the Paths"""
